﻿using Khronos;
using OpenGL;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GameApp
{
    public partial class MainForm : Form
    {
        DisplayManager _displayManager = new DisplayManager();

        RawModel _model;
        Loader _loader;
        Renderer _renderer;
        StaticShader _shader;

        float[] _vertices =
        {
            -0.5f, 0.5f, 0f,   // V0
            -0.5f, -0.5f, 0f,  // V1
            0.5f, -0.5f, 0f,   // V2
            0.5f, 0.5f, 0f   // V3
        };

        int[] _indices =
        {
            0, 1, 3,  // Top left triangle (V0, V1, V3)
            3, 1, 2   // Bottom right triangle (V3, V1, V2)
        };

        public MainForm()
        {
            InitializeComponent();
        }

        private void glControl_ContextCreated(object sender, OpenGL.GlControlEventArgs e)
        {
            GlControl glControl = (GlControl)sender;
            _displayManager.createDisplay(glControl);

            _loader = new Loader();
            _renderer = new Renderer();
            _shader = new StaticShader();
            _model = _loader.loadToVAO(_vertices, _indices);
        }

        private void glControl_ContextUpdate(object sender, GlControlEventArgs e)
        {
        }

        private void glControl_ContextDestroying(object sender, GlControlEventArgs e)
        {
            _loader.CleanUp();
            _shader.CleanUp();
        }

        private void glControl_Render(object sender, OpenGL.GlControlEventArgs e)
        {
            Control senderControl = (Control)sender;
            Gl.Viewport(0, 0, senderControl.ClientSize.Width, senderControl.ClientSize.Height);

            _renderer.Prepare();
            _shader.Start();
            _renderer.Render(_model);
            _shader.Stop();

            _displayManager.updateDisplay();
        }

    }
}
